<template>
  <div class="station-card-list">
    <div class="plan">
      <el-button class="btn-roll-call-plan" type="primary" @click="toPlan" v-if="false">点名计划</el-button>
    </div>
    <div class="list-container">
      <div
        v-for="item in stationTypeList"
        class="card"
        :class="[item.type === 3 || item.type === 4 ? 'card-mask' : '']"
        :key="item.code"
      >
        <!-- 点名中 -->
        <div class="card-calling" v-if="item.type === 3">
          <div class="card-calling-icon"></div>
          <div class="card-calling-text card-calling-text-progress">点名中...</div>
        </div>
        <!-- 点名完成 -->
        <div class="card-calling" v-if="item.type === 4">
          <div class="card-calling-success"></div>
          <div class="card-calling-text card-calling-text-success">点名完成</div>
        </div>
        <div class="card-top">
          <img class="type-icon" :src="bindIcon(item)" alt="" />
          <div class="type-name">{{ item.name }}</div>
          <div class="type-button">
            <el-button type="primary" plain size="mini" @click.stop="toStationRollCallList(item)">查看记录</el-button>
          </div>
        </div>

        <div class="card-num">
          <div class="card-num-sub" style="border-right: 1px solid #cfe2ff; opacity: 1">
            <span class="card-num-sum">{{ item.countOfStations }}</span
            ><span class="card-num-text">个</span>
          </div>
          <div class="card-num-sub">
            <span class="card-num-sum">{{ item.countOfSecurityMen }}</span
            ><span class="card-num-text">人</span>
          </div>
          <div class="card-num-sub" style="border-right: 1px solid #cfe2ff; opacity: 1">
            <span class="card-num-text">总数</span>
          </div>
          <div class="card-num-sub">
            <span class="card-num-text">总人数</span>
          </div>
        </div>
        <div class="card-btn">
          <el-button class="btn-roll-card-select" type="primary" plain @click.stop="selectStation(item.code)">选择</el-button>
          <el-button class="btn-roll-card-roll" type="primary" @click.stop="stationRollCalls(item.code)">点名</el-button>
        </div>
      </div>
    </div>
    <el-dialog title="选择驻勤点" :visible.sync="dialogVisible" width="1361px">
      <el-form :inline="true" :model="queryForm" label-width="125px">
        <el-form-item label="驻勤点名称">
          <el-input style="width: 200px" v-model="queryForm.stationName" @keydown.enter.native="searchStations"></el-input>
        </el-form-item>
        <el-form-item label="保安服务公司名称" v-if="isSupervision">
          <el-input style="width: 200px" v-model="queryForm.companyName" @keydown.enter.native="searchStations"></el-input>
        </el-form-item>
        <el-form-item label="辖区" v-if="isSupervision">
          <tree-select
            class="tree-select"
            :options="departs"
            placeholder="请选择"
            :normalizer="normalizer"
            v-model="queryForm.depart"
            noOptionsText="无数据"
            noResultsText="无数据"
            @select="onSelectDepart"
          />
        </el-form-item>
        <el-form-item class="transfer-div">
          <el-transfer
            class="transfer"
            v-model="yesData"
            :props="{ key: 'id', label: 'name' }"
            :titles="['全部', '已选']"
            :data="stationList"
            @change="handleChange"
            :render-content="renderFunc"
          >
          </el-transfer>
        </el-form-item>
        <div class="footer">
          <el-button type="primary" @click="confirmStation">确认</el-button>
          <el-button type="primary" plain @click="clossDialog">关闭</el-button>
        </div>
      </el-form>
    </el-dialog>
  </div>
</template>

<script>
import treeSelect from "@riophae/vue-treeselect";
import "@riophae/vue-treeselect/dist/vue-treeselect.css";
import { getAllStationTypes, getStationStatistics, getBasicStations } from "@/api/security-station-controller";
import { getDeparts } from "@/api/departs-controller";
import { getRollCallStatus, rollCalls } from "@/api/roll-call-controller";
import _ from "lodash";

export default {
  name: "stationCardList",
  components: {
    treeSelect,
  },
  methods: {
    async init() {
      this.departs = await getDeparts({ deskTypes: this.deskTypes }); // 获取辖区树
      console.log("departs", this.departs);
      this.stationTypeList = await getAllStationTypes({ deskTypes: [] }); //获取驻勤点类型信息
      this.stationTypeList.forEach((item) => {
        this.stationIdsToType[item.code + ""] = [];
        item.type = 1; // 初始化状态   3 点名中  4 点名完成
        item.callTime = 0;
      });
      console.log("stationIdsToType", this.stationIdsToType);
      await getStationStatistics({
        pageIndex: 0,
        pageSize: 0,
        forExport: true,
        stationIds: [],
      }).then((stationStatistics) => {
        console.log("stationStatistics", stationStatistics);
        this.stationTypeList.forEach((item) => {
          item["countOfSecurityMen"] = 0;
          item["countOfStations"] = 0;
          if (stationStatistics) {
            stationStatistics.forEach((statistic) => {
              if (item.code === statistic.stationTypeCode) {
                item["countOfSecurityMen"] = statistic.countOfSecurityMen;
                item["countOfStations"] = statistic.countOfStations;
              }
            });
          }
        });
        this.$forceUpdate();
        console.log("stationTypeList", this.stationTypeList);
      });

      const userInfo = localStorage.getItem("userInfo");
      let userInfoJson;
      if (userInfo) {
        userInfoJson = JSON.parse(userInfo);
      }
      if (userInfoJson) {
        //判断是否是企业或内保单位
        if (userInfoJson.role === "Company" || userInfoJson.role === "Proprietor") {
          this.isCompany = true;
        }
        this.isSupervision = JSON.parse(userInfo).superviseDesk; //判断身份 ture:监管账户；false:企业账户
      }
    },
    toPlan() {
      this.$emit("toPlan");
    },
    toStationRollCallList(data) {
      this.$emit("toStationRollCallList", { data });
    },

    //选中某个驻勤点类型
    selectStation(stationTypeCode) {
      this.selectedStationType = stationTypeCode;
      this.searchStations();
      this.yesData = this.stationIdsToType[this.selectedStationType];
    },
    //查询驻勤点
    searchStations() {
      getBasicStations({
        pageIndex: 1,
        pageSize: 1000,
        forExport: true,
        deskTypes: this.isCompany ? ["Normal", "Proprietor"] : this.deskTypes,
        stationTypes: [this.selectedStationType + ""],
        name: this.queryForm.stationName,
        companyName: this.queryForm.companyName,
        superviseDepartIds: this.queryForm.superviseDepartIds,
      })
        .then((res) => {
          console.log("stationList", res);
          this.stationList = res.data;
        })
        .catch((err) => {
          console.log(err);
        });
      this.dialogVisible = true;
    },
    //右侧数据变动触发函数
    handleChange(value, direction, movedKeys) {
      console.log(value, direction, movedKeys);
      this.stationIdList = movedKeys; //文件列表的id数组
    },
    confirmStation() {
      this.stationIdsToType[this.selectedStationType] = this.yesData;
      console.log("confirmStation", this.stationIdsToType[this.selectedStationType]);
      this.dialogVisible = false;
      //更新选中驻勤点数据
      getStationStatistics({
        pageIndex: 0,
        pageSize: 0,
        forExport: true,
        stationIds: this.yesData,
        stationTypeCode: [this.selectedStationType],
      }).then((stationStatistics) => {
        console.log("stationStatistics", stationStatistics);
        this.stationTypeList.forEach((item) => {
          if (stationStatistics) {
            stationStatistics.forEach((statistic) => {
              if (item.code === statistic.stationTypeCode) {
                item["countOfSecurityMen"] = statistic.countOfSecurityMen;
                item["countOfStations"] = statistic.countOfStations;
              }
            });
          }
        });
        this.$forceUpdate();
        console.log("stationTypeList", this.stationTypeList);
      });
    },
    onSelectDepart(node) {
      console.log(node);
      if (node?.children?.length > 0) {
        const getTail = (item) => (item.children && item.children.length > 0 ? item.children.map((m) => getTail(m)) : [item]);
        const result = _.flattenDeep(node.children.map((m) => [m, ...getTail(m)]));
        console.log(" log result", result);
        this.queryForm.superviseDepartIds = [node.code, ...result.map((x) => x.code)];
      } else {
        this.queryForm.superviseDepartIds = [node.code];
      }
      this.selectStations();
    },
    clossDialog() {
      this.dialogVisible = false;
    },
    bindIcon({ name }) {
      return require("@/assets/images/rollManage/" + name + ".png");
    },
    //点名
    stationRollCalls(stationTypeCode) {
      rollCalls({
        stationTypes: [stationTypeCode + ""],
        securityStationIds: this.stationIdsToType[stationTypeCode],
      })
        .then((res) => {
          this.stationTypeList.forEach((stationType) => {
            if (stationType.code === stationTypeCode) {
              stationType.type = 3;
              stationType.callTime = 0;
            }
          });
          this.$forceUpdate();

          const { rollCallId } = res;
          this.stationTypeList.forEach((stationType) => {
            // station.rollCallId = "";
            if (stationType.code === stationTypeCode) {
              stationType.rollCallId = rollCallId;
            }
          });
          // 更新点名中的ids数组，已存在的就不加了
          if (!this.rollCallIds.includes(rollCallId)) {
            this.rollCallIds.push(rollCallId);
          }
          // 轮询状态，如果存在定时器说明之前点过一种驻勤点了，再点一次重新开始定时器避免定时器错乱
          if (this.statusTimer) clearInterval(this.statusTimer);
          this.askStatus();
          this.statusTimer = setInterval(() => {
            this.askStatus();
          }, 3000); // 3s轮询一次
        })
        .catch((err) => {
          this.$store.commit("showMessage", {
            type: "error",
            message: err.response.data.message,
            duration: 0,
          });
          console.log(err);
        });
    },
    async askStatus() {
      // 如果所有点名状态都完成且有定时器存在-->不再需要定时器
      if (this.rollCallIds.length <= 0 && this.statusTimer) {
        clearInterval(this.statusTimer);
        this.statusTimer = null;
        return false; // 直接中断
      }
      const res = await getRollCallStatus({ rollCallIds: this.rollCallIds });
      if (res && res.length > 0) {
        res.forEach((rollCall) => {
          const stationType = this.stationTypeList.find((s) => s.rollCallId === rollCall.rollCallId);
          if (stationType) {
            console.log(stationType);
            // 2、下一个三秒去掉遮罩(状态初始化)，从ids数组中去掉这个id
            if (stationType.type === 4) {
              stationType.type = 1;
              const index = this.rollCallIds.indexOf(rollCall.rollCallId);
              if (index !== -1) {
                this.rollCallIds.splice(index, 1);
              }
            } else {
              stationType.type = rollCall.completed ? 4 : 3; // 1、先把状态变成完成，但还是去查一次状态
            }

            if (rollCall.completed) {
              stationType.callTime = 0; // 清空计时
            } else {
              stationType.callTime = stationType.callTime + 3; // 还没接就一直计时 3s轮询一次
              if (stationType.callTime >= 180) {
                // 如果超过3分钟
                stationType.type = 1;
                const index = this.rollCallIds.indexOf(rollCall.rollCallId);
                if (index !== -1) {
                  this.rollCallIds.splice(index, 1);
                }
              }
            }
            this.$forceUpdate();
          }
        });
      }
    },
  },
  data() {
    return {
      platform: localStorage.getItem("platform") || "1",
      deskTypes:
        localStorage.getItem("platform") === "2"
          ? ["Proprietor"]
          : localStorage.getItem("platform") === "3"
          ? ["TemporarySecurity"]
          : ["Normal"],
      isCompany: false,
      isSupervision: false, //是否监管端 显示辖区：企业账号进来不展示【辖区】筛选项:false，只有监管账号进来才展示:true
      selectedStationType: "", //当前选中的驻勤点类型
      queryForm: {
        stationName: "", //驻勤点名称
        companyName: "", //公司名称
        depart: null, //辖区
        superviseDepartIds: [],
      },
      stationTypeList: [], //驻勤点类型
      stationList: [], //驻勤点查询结果
      stationIdList: [], //移动驻勤点
      stationIdsToType: {}, //驻勤点类型对应的选中驻勤点id数组
      yesData: [], // 注意:key 的字符类型要一致!!!
      renderFunc(h, option) {
        return (
          <span title={option.name}>
            {option.companyName}-{option.name}
          </span>
        );
      },
      dialogVisible: false,
      departs: [],
      normalizer(node) {
        if (node.children && !node.children.length) {
          delete node.children;
        }
        return {
          id: node.code,
          label: node.name,
          children: node.children,
          // isDisabled: !node.selectable,
          isDisabled: false,
        };
      },
      rollCallIds: [],
      statusTimer: null,
    };
  },
  mounted() {
    this.init();
  },
  destroyed() {
    if (this.statusTimer) clearInterval(this.statusTimer);
    this.statusTimer = null;
  },
};
</script>

<style scoped lang="scss">
.tree-select {
  //padding-top: 6px;
  width: 350px;
  // font-size: 14px;
  line-height: 35px;
  // font-family: PingFang SC-Regular, PingFang SC;
  // font-weight: 400;
  color: #606266;
  .vue-treeselect__placeholder {
    line-height: 35px;
  }
  .vue-treeselect__control {
    height: 35px !important;
  }
  .vue-treeselect__placeholder,
  .vue-treeselect__single-value {
    line-height: 35px;
  }
}
.transfer-div {
  width: 100%;
  display: flex;
  justify-content: center;
}

.station-card-list {
  width: 100%;
  height: 100%;
  .plan {
    margin-bottom: 28px;
    .btn-roll-call-plan {
      // height: 39px;
    }
  }
  .list-container {
    width: 100%;
    height: calc(100% - 39px - 28px);
    overflow-x: hidden;
    overflow-y: auto;
    display: grid;
    grid-template-columns: repeat(4, 1fr);
    .card {
      width: 362px;
      height: 260px;
      // cursor: pointer;
      background: linear-gradient(180deg, #ecf3ff 0%, #ffffff 100%);
      box-shadow: 10px 10px 20px 0 rgba(46, 78, 134, 0.16);
      border-radius: 4px;
      border: 3px solid;
      border-image: linear-gradient(180deg, rgba(255, 255, 255, 1), rgba(255, 255, 255, 0)) 3 3;
      margin-bottom: 48px;
      position: relative;
      &-mask {
        border: 0;
        &::after {
          background: rgba(0, 0, 0, 0.3);
          backdrop-filter: blur(3px);
          display: block;
          width: 100%;
          height: 100%;
          border-radius: 4px;
          content: " ";
          position: absolute;
          top: 0;
          left: 0;
          z-index: 1;
        }
      }
      &-calling {
        width: 100%;
        height: 100%;
        position: absolute;
        top: 0;
        left: 0;
        z-index: 2;
        display: flex;
        align-items: center;
        justify-content: center;
        flex-direction: column;
        &-icon {
          width: 80px;
          height: 80px;
          background: url("~@/assets/images/rollCall/loading.png") no-repeat;
          background-size: 100% 100%;
          margin-bottom: 17px;
        }
        &-success {
          width: 95px;
          height: 95px;
          /*TODO 修改这个点名成功图标*/
          background: url("~@/assets/images/rollCall/success.png") no-repeat;
          background-size: 100% 100%;
          margin-bottom: 17px;
        }
        &-text {
          font-weight: 400;
          font-size: 20px;
          &-progress {
            color: #ffffff;
          }
          &-success {
            color: #e2ffd0;
          }
        }
      }
      .card-num {
        width: 362px;
        text-align: center; //实现水平居中
        position: absolute;
        left: 0;
        //margin: auto;
        right: 0;
        margin-top: 92px;
        float: left;
        .card-num-sub {
          width: 180px;
          float: left;
          .card-num-sum {
            font-size: 28px;
            font-family: Source Han Sans CN-Medium, Source Han Sans CN, serif;
            font-weight: 500;
            color: #333333;
          }
          .card-num-text {
            width: 16px;
            height: 16px;
            font-size: 16px;
            font-family: Source Han Sans CN-Regular, Source Han Sans CN, serif;
            font-weight: 400;
            color: #666666;
          }
        }
      }
      .card-top {
        margin-top: 0;
        .type-icon {
          width: 36px;
          height: 36px;
          margin-left: 24px;
          float: left;
          margin-top: 10px;
        }
        .type-name {
          width: 200px;
          height: 36px;
          line-height: 36px;
          font-size: 20px;
          font-weight: 500;
          color: #3d3d3d;
          padding-left: 10px;
          float: left;
          margin-top: 10px;
        }
        .type-button {
          position: absolute;
          right: 10px;
          top: 10px;
          z-index: 9;
        }
      }
      .card-btn {
        text-align: center; //实现水平居中
        position: absolute;
        left: 0;
        margin: auto;
        right: 0;
        .btn-roll-card-select {
          width: 112px;
          height: 36px;
          line-height: 12px;
          margin-top: 204px;
        }

        .btn-roll-card-roll {
          width: 112px;
          height: 36px;
          line-height: 12px;
          margin-top: 204px;
          background-color: #0e70ff;
        }
      }
    }
  }
}
</style>
<style>
.el-transfer-panel {
  width: 550px;
  height: 320px;
}
.el-transfer-panel__list.is-filterable {
  height: 400px;
}
.footer {
  display: flex;
  justify-content: center;
  margin-top: 10px;
}
</style>
